# Name :          All Faces Coloured
# Description :   Colours all faces with a random selection of the named
#   			        Sketchup colours
# Author[s] :     CJ
#
# TODO:           Ensure random colours are different (random start point, incremental)
#                 Improve UI (Icons, submenu)
#                 Future: Improve options for groups & components to be single colour 
#                 Future: Apply to selection only if there is a selection set
#
# 1.0  (20.01.2015) - Version 1
#-------------------------------------------------------
require 'sketchup.rb'
#-------------------------------------------------------

module CJ_ColourFaces

  ### Add to Plugins menu on startup

  unless file_loaded?( __FILE__ )
    menu = UI.menu( 'Plugins' )
    menu.add_item( 'Colour Faces' ) { self.initialise }
    menu.add_item( 'Reset Faces' ) { self.initialise(true) }
    menu.add_item( 'Colour Faces same' ) { self.initialise(false, true) }
  end

  ### Initialise & run plugin

  def self.initialise(reset=false, same=false)

    @colour_selection = Sketchup::Color.names
    @colour_number = @colour_selection.length
    @colour_current = @colour_selection[rand(@colour_number)]
    @reset = reset
    @same = same

    @model = Sketchup.active_model

    @model.start_operation("Colour faces", true)

    @entities = Sketchup.active_model.entities
    self.colourfaces(@entities)

    @model.commit_operation

  end

  ### Main method to recursively colour all faces

  def self.colourfaces(entities=[])

    entities.each do |entity|
      if entity.is_a? Sketchup::Face
        if entity.material == nil && !@reset
          entity.material = @colour_current
          self.softcheck(entity)
          @colour_current = @colour_selection[rand(@colour_number)] unless @same
        elsif @reset
          entity.material = nil
        else next
        end
      elsif entity.is_a? Sketchup::Group
          ents = entity.entities
          @colour_current = @colour_selection[rand(@colour_number)] if @same
          self.colourfaces(ents)
      elsif entity.is_a? Sketchup::ComponentInstance
          ents = entity.definition.entities
          @colour_current = @colour_selection[rand(@colour_number)] if @same
          self.colourfaces(ents)
      end
    end
  end # self.colourfaces

  ### Method to check for adjoined softened faces & colour them

  def self.softcheck(face)
    face.edges.each do |edge|
      if edge.soft?
      @face_array = edge.faces 
      @face_array.delete(face)
        if @face_array[0].material == nil
          @face_array[0].material = @colour_current
          self.softcheck(@face_array[0])
        end
      else next
      end
    end
  end

end # CJ_ColourFaces